Object subclass:#PrintConverter 
	 instanceVariableNames:'valueToStringBlock stringToValueBlock'
	 classVariableNames:''
	 poolDictionaries:''
	 category:'Interface-Support'
!

!PrintConverter class methodsFor:'documentation'!

version 
"
$Header: /cvs/stx/stx/libview2/Attic/PrintConv.st,v 1.3 1995-05-09 00:21:50 claus Exp $
"
!

documentation
"
    printConverters are used with editFields to convert an object
    to/from some printed representation.
    Conversion is done via two blocks which can be set at instance
    creation time - either as custom blocks ot to one of the
    standard conversions.

    Notice: this class was implemented using protocol information
    from alpha testers - it may not be complete or compatible to
    the corresponding ST-80 class. If you encounter any incompatibilities,
    please forward a note to the ST/X team.
"
!

examples 
"
    stupid examples"
      |conv|

      conv := (PrintConverter new)
		  toPrint:[:date | date printString]
		  toRead:[:string | Date readFromString:string].
      (conv printStringFor:(Date today)) inspect.
      (conv readValueFrom:(Date today printString)) inspect


      |conv|

      conv := (PrintConverter new) initForNumber.
      (conv printStringFor:12345) inspect.
      (conv readValueFrom:'12345') inspect


      |conv|

      conv := (PrintConverter new) initForYesNo.
      (conv printStringFor:true).  
      (conv printStringFor:false).    
      (conv readValueFrom:'yes').  
      (conv readValueFrom:'no').  
      'if language is german:'.
      (conv readValueFrom:'ja').    
      (conv readValueFrom:'nein')  

    concrete example: convert in an inputField:

      |dialog field|

      dialog := Dialog new.
      dialog addTextLabel:'a number (and only numbers):'.
      dialog addVerticalSpace.
      field := dialog addInputFieldOn:(0 asValue).
      field converter:(PrintConverter new initForNumber).
      field immediateAccept:true.
      dialog addOkButton.
      dialog open.
      dialog accepted ifTrue:[
	  Transcript showCr:field editValue
      ]


    see more examples in the EditField examples.
"
! !

!PrintConverter class methodsFor:'instance creation'!

new
    ^ (super new) 
	toPrint:[:val | val]
	toRead:[:string | string]
! !

!PrintConverter methodsFor:'initialization'!

toPrint:printBlock toRead:readBlock
    "initialize to convert using two custom blocks"

    valueToStringBlock := printBlock.
    stringToValueBlock := readBlock.
!

initForInteger
    "initialize to convert to/from an integer 
     - if the string is empty or invalid, convert to 0"

    valueToStringBlock := [:num | num printString].
    stringToValueBlock := [:string | Integer readFromString:string onError:0]
!

initForIntegerOrNil
    "initialize to convert to/from an integer 
     - if the string is empty or invalid, convert to nil"

    valueToStringBlock := [:num | 
	num isNil ifTrue:[''] 
		  ifFalse:[num printString]].
    stringToValueBlock := [:string | 
	Integer readFromString:string onError:nil]
!

initForNumber
    "initialize to convert to/from a number 
     - if the string is empty or invalid, convert to 0"

    valueToStringBlock := [:num | num printString].
    stringToValueBlock := [:string | Number readFromString:string onError:0]
!

initForNumberOrNil
    "initialize to convert to/from a number 
     - if the string is empty or invalid, convert to nil"

    valueToStringBlock := [:num | 
	num isNil ifTrue:[''] 
		  ifFalse:[num printString]].
    stringToValueBlock := [:string | 
	Number readFromString:string onError:nil]
!

initForString
    "initialize for a string - this is trivial"

    valueToStringBlock := [:val | val isNil ifTrue:[''] ifFalse:[val]].
    stringToValueBlock := [:string | string]
!

initForDate
    "initialize to convert to/from a date 
     - if the string is empty or invalid, convert to the current date"

    valueToStringBlock := [:date | date printString].
    stringToValueBlock := [:string | Date readFromString:string onError:[Date today]]
!

initForDateOrNil
    "initialize to convert to/from a date 
     - if the string is empty or invalid, convert to nil"

    valueToStringBlock := [:date | date printString].
    stringToValueBlock := [:string | Date readFromString:string onError:nil]
!

initForYesNo
    "initialize to convert a 'yes'/'no' string to/from a boolean 
     The string is supposed to be in the current Language
     (i.e. if german, ja/nein has to be enterred.
     Invalid entries are converted to false."

    valueToStringBlock := [:bool | bool ifTrue:[ApplicationModel classResources string:'yes']
					ifFalse:[ApplicationModel classResources string:'no']].
    stringToValueBlock := [:string | string = (ApplicationModel classResources string:'yes')]
! !

!PrintConverter methodsFor:'converting'!

printStringFor:aValue
    ^ valueToStringBlock value:aValue
!

readValueFrom:aString
    ^ stringToValueBlock value:aString
! !
